package holeg.preferences;

import java.awt.Color;

import holeg.model.Flexibility.FlexState;
import holeg.ui.model.DecoratedHolonObject.HolonObjectState;

/**
 * A Class to save all colors in Holeg.
 * @author Tom
 *
 */
public class ColorPreference {
	
	
	public static class Element{
		public static class Priority{
			public static final Color Essential = new Color(201, 44, 74);
			public static final Color High = new Color(255, 92, 26);
			public static final Color Medium = new Color(255, 216, 57);
			public static final Color Low = new Color(255, 255, 175);
			public static final Color NoData = new Color(21, 8, 17);
		}
		public static final Color Active = new Color(255, 203, 35);
		public static final Color Inactive = new Color(128, 154, 163);
	}
	public static class HolonObject{
		public static final Color Producer = new Color(153, 204, 255);
		public static final Color OverSupplied = new Color(166, 78, 229);
		public static final Color Supplied = new Color(13, 175, 28);
		public static final Color PartiallySupplied = Color.yellow;
		public static final Color NotSupplied = new Color(230, 120, 100);
		public static final Color NoEnergy = Color.white;
	
		public static Color getStateColor(HolonObjectState state) {
			switch (state) {
			case NOT_SUPPLIED:
				return NotSupplied;
			case OVER_SUPPLIED:
				return OverSupplied;
			case PARTIALLY_SUPPLIED:
				return PartiallySupplied;
			case PRODUCER:
				return Producer;
			case SUPPLIED:
				return Supplied;
			case NO_ENERGY:
			default:
				return NoEnergy;
			}
		}
	}
	
	public static class Edge{
		public static final Color Working = new Color(13, 175, 28);
		public static final Color Burned = Color.red;
	}
	
	
	
	public static class Energy{
		public static final Color Production = new Color(61, 133, 243);
		public static final Color Consumption = new Color(255, 67, 60);
	}
	public static class Flexibility{
		public static final Color InUse = new Color(182, 238, 166);
		public static final Color OnCooldown = new Color(239, 215, 128);
		public static final Color Offered = new Color(75, 170, 72);
		public static final Color NotOffered = new Color(237, 106, 90);
		public static final Color Unavailable = new Color(193, 193, 193);
		
		public static Color getStateColor(FlexState state) {
			switch(state) {
			case IN_USE:
				return InUse;
			case NOT_OFFERED:
				return NotOffered;
			case OFFERED:
				return Offered;
			case ON_COOLDOWN:
				return OnCooldown;
			case UNAVAILABLE:
			default:
				return Unavailable;
			}
		}
	}
	
	public static class Panel {
		public static final Color Transparent = new Color(0,0,0,0);
		public static final Color ObjectSelection = new Color(128, 174, 247, 40);
		public static final Color ObjectSelectionBorder = Color.lightGray;
		public static final Color MouseSelection = Color.blue;
		public static final Color Background = new Color(250, 250, 250);
		public static final Color Title = new Color(54, 73, 78);
	}
	
	public static class Dialog{
		public static final Color BackgroundColor = new Color(255, 50, 50);
	}
	
	public static class Category {
		public final static Color Focus = new Color(0, 0, 255);	
	}
	public static class Inspector{
		public final static Color Selected = new Color(126, 186, 255);
		public final static Color Border = new Color(171, 173, 179);
	}
	
	public static class GUI {
		//TODO(Tom2021-12-1) for what is this collor used
		public static final Color PALE_RED = new Color(255, 192, 192);
	}
}
