package holeg.ui.view.canvas;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Component;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.event.MouseEvent;
import java.util.Optional;
import java.util.logging.Logger;

import holeg.model.AbstractCanvasObject;
import holeg.model.Edge;
import holeg.model.GroupNode;
import holeg.model.HolonObject;
import holeg.model.HolonSwitch;
import holeg.model.Node;
import holeg.preferences.ColorPreference;
import holeg.ui.controller.Control;
import holeg.ui.model.Consumer;
import holeg.ui.model.DecoratedGroupNode;
import holeg.ui.model.DecoratedSwitch;
import holeg.ui.model.ExitCable;
import holeg.ui.model.GuiSettings;
import holeg.ui.model.Model;
import holeg.ui.model.Passiv;
import holeg.ui.model.Supplier;
import holeg.ui.model.VisualRepresentationalState;
import holeg.ui.view.inspector.UnitGraph;
import holeg.utility.ImageImport;
import holeg.utility.Vector2Int;

//TODO(Tom2021-12-1) delete GroupNodeCanvas completely and only have canvas Class
public class GroupNodeCanvas extends Canvas {
	private static final Logger log = Logger.getLogger(GroupNodeCanvas.class.getName());
	private String parentPath;
	private Component parentComponent;

	public GroupNodeCanvas(Model mod, Control control, UnitGraph unitGraph, GroupNode groupNode, String parentPath,
			Component parentComponent) {
		super(mod, control, unitGraph);
		this.groupNode = groupNode;
		this.parentPath = parentPath;
		this.parentComponent = parentComponent;
	}

	public void setGroupNode(GroupNode upperNode) {
		this.groupNode = upperNode;
	}

	public GroupNode getGroupNode() {
		return groupNode;
	}

	public String getParentPath() {
		return parentPath;
	}

	public Component getParentComponent() {
		return parentComponent;
	}

	public void paintComponent(Graphics g) {
		// super.paintComponent(g);
		Graphics2D g2d = (Graphics2D) g;
		g2d.clearRect(0, 0, this.getWidth(), this.getHeight());
		g2d.setRenderingHints(new RenderingHints(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON));
		// -->Old code
		if (drawEdge) {
			g2d.setColor(Color.BLACK);
			g2d.setStroke(new BasicStroke(1));
			g2d.drawLine(tempCps.getPosition().getX(), tempCps.getPosition().getY(), x, y);
		}
		// <--
		// SelectedCable


		Optional<VisualRepresentationalState> visState = control.getSimManager()
				.getActualVisualRepresentationalState();
		if (visState.isEmpty()) {
			return;
		}
		DecoratedGroupNode actualGroupNode = visState.get().getCreatedGroupNodes().get(groupNode);
		// VisualState Representation:
		for (ExitCable cable : actualGroupNode.getExitCableList()) {
			paintExitCable(g2d, cable);
		}
		for (Edge cable : actualGroupNode.getInternCableList()) {
			paintCable(g2d, cable, GuiSettings.getSelectedEdges().contains(cable));
		}

		for (DecoratedGroupNode dGroupNode : actualGroupNode.getGroupNodeList()) {
			paintGroupNode(g2d, dGroupNode);
		}
		for (Consumer con : actualGroupNode.getConsumerList()) {
			paintConsumer(g2d, con);
		}
		for (Supplier sup : actualGroupNode.getSupplierList()) {
			paintSupplier(g2d, sup);
		}
		for (Passiv pas : actualGroupNode.getPassivList()) {
			paintCanvasObject(g2d, pas);
		}
		for (DecoratedSwitch dSwitch : actualGroupNode.getSwitchList()) {
			paintSwitch(g2d, dSwitch);
		}
		for (Node node : actualGroupNode.getNodeList()) {
			drawCanvasObject(g2d, "/Images/node.png", node.getPosition());
		}

		// -->oldCode
		if (doMark) {
			g2d.setColor(Color.BLACK);
			g2d.setStroke(new BasicStroke(0));
			drawMarker(g2d);
		}
		// Test Selection
		// Objects:
		g2d.setStroke(new BasicStroke(1));
		Color transparentGrey = ColorPreference.Panel.ObjectSelection;
		for (AbstractCanvasObject aCps : GuiSettings.getSelectedObjects()) {
			if (aCps instanceof Node) {
				Vector2Int pos = aCps.getPosition();
				g2d.setColor(transparentGrey);
				g2d.fillOval(pos.getX() - (int) (GuiSettings.getPictureScaleDiv2()),
						pos.getY() - (int) (GuiSettings.getPictureScaleDiv2()), GuiSettings.getPictureScale(), GuiSettings.getPictureScale());
				g2d.setColor(ColorPreference.Panel.ObjectSelectionBorder);
				g2d.setStroke(new BasicStroke(2));
				g2d.drawOval(pos.getX() - (int) (GuiSettings.getPictureScaleDiv2()),
						pos.getY() - (int) (GuiSettings.getPictureScaleDiv2()), GuiSettings.getPictureScale(), GuiSettings.getPictureScale());
			} else {
				Vector2Int pos = aCps.getPosition();
				g2d.setColor(transparentGrey);
				g2d.fillRect(pos.getX() - (int) (GuiSettings.getPictureScaleDiv2() * 1.5f),
						pos.getY() - (int) (GuiSettings.getPictureScaleDiv2() * 1.5f), (int) (GuiSettings.getPictureScale() * 1.5f),
						(int) (GuiSettings.getPictureScale() * 1.5f));
				g2d.setColor(ColorPreference.Panel.ObjectSelectionBorder);
				g2d.setStroke(new BasicStroke(2));
				g2d.drawRect(pos.getX() - (int) (GuiSettings.getPictureScaleDiv2() * 1.5f),
						pos.getY() - (int) (GuiSettings.getPictureScaleDiv2() * 1.5f), (int) (GuiSettings.getPictureScale() * 1.5f),
						(int) (GuiSettings.getPictureScale() * 1.5f));
			}

		}
		// maybeReplace:
		if (mayBeReplaced != null) {
			Vector2Int pos = mayBeReplaced.getPosition();
			g.drawImage(ImageImport.loadImage("/Images/replace.png"), pos.getX() + GuiSettings.getPictureScaleDiv2(),
					pos.getY() - GuiSettings.getPictureScale(), GuiSettings.getPictureScaleDiv2(), GuiSettings.getPictureScaleDiv2(), null);
		}
		// <-- OldCode
	}

	@Override
	public void mousePressed(MouseEvent e) {
		this.grabFocus();
		if (!disabled) {
			tempCps = null;
			dataSelected = null;
			edgeHighlight = null;
			GuiSettings.getSelectedEdges().clear();
			// Object Selection

			if (e.getX() > 0) {
				for (AbstractCanvasObject cps : groupNode.getObjectsInThisLayer().toList()) {
					cx = cps.getPosition().getX() - GuiSettings.getPictureScaleDiv2();
					cy = cps.getPosition().getY() - GuiSettings.getPictureScaleDiv2();
					if (x - GuiSettings.getPictureScale() <= cx && y - GuiSettings.getPictureScale() <= cy && x >= cx && y >= cy) {
						tempCps = cps;

						dragging = true;
						if (e.isControlDown() && tempCps != null) {
							log.info("Add Remove Selection GROUPNODECANVAS");
							if (GuiSettings.getSelectedObjects().contains(tempCps)) {
								control.removeObjectFromSelection(tempCps);
							} else {
								control.addSelectedObject(tempCps);
								if (tempCps instanceof GroupNode)
									control.getObjectsInDepth();
							}
						}

						// If drawing an Edge (CTRL down)
						if (tempCps.getClass() == HolonObject.class) {
							HolonObject tempObj = ((HolonObject) tempCps);
							dataSelected = tempObj.getElements();
						}
						if (e.isShiftDown()) {
							drawEdge = true;
							dragging = false;
						}
						break;
					}
				}
			}
			// Edge Selection
			if (e.getButton() == MouseEvent.BUTTON1) {
				if (tempCps == null) {
					edgeHighlight = mousePositionOnEdge(x, y);
					GuiSettings.getSelectedEdges().add(edgeHighlight);
					if (!e.isControlDown() && e.getButton() != MouseEvent.BUTTON3) {
						control.clearSelection();
					}
				}

				if (edgeHighlight == null && tempCps == null) {
					sx = e.getX();
					sy = e.getY();
					doMark = true;
				}
				repaint();
			}
		}
	}

	@Override
	public void mouseReleased(MouseEvent e) {
		if (!disabled) {
			x = e.getX();
			y = e.getY();
			dragging = false;

			if (drawEdge) {
				drawEdge = false;
				drawDeleteEdge();
			}

			if (dragged) {
				control.tryAutoSave();

				/**
				 * check if tempCps could replace an Object on the UpperNodeanvas
				 */
				if (GuiSettings.getSelectedObjects().size() == 1 && checkForReplacement(groupNode.getObjectsInThisLayer().toList(), tempCps,
						tempCps.getPosition().getX(), tempCps.getPosition().getY())) {
					/**
					 * if UpperNode would be replaced, close its tabs
					 */
					if (mayBeReplaced instanceof GroupNode)
						closeUpperNodeTab(mayBeReplaced.getId());
					/**
					 * replace on canvas
					 */
					control.replaceObjectInGroupNode(mayBeReplaced, tempCps, groupNode);
					mayBeReplaced = null;
				}
			}

			if (!e.isControlDown() && !dragged && tempCps != null && MouseEvent.BUTTON3 != e.getButton()) {
				control.clearSelection();
				control.addSelectedObject(tempCps);
				if (tempCps instanceof GroupNode)
					control.getObjectsInDepth();
			}

			dragged = false;

			// Rightclick List
			setRightClickMenu(e);

			markObjects();
			boolean doubleclick = doubleClick();
			if (doubleclick && tempCps != null && tempCps instanceof HolonSwitch sw) {
				sw.switchState();
			}
			if (doubleclick && tempCps != null && tempCps instanceof GroupNode groupnode) {
				control.getGui().openNewUpperNodeTab(groupnode);
			}
			control.calculateStateAndVisualForTimeStep(model.getCurrentIteration());
			repaint();
		}
	}

	@Override
	public void mouseDragged(MouseEvent e) {
		if (!disabled) {
			// If Edge is drawn
			x = e.getX();
			y = e.getY();
			if (!GuiSettings.getSelectedObjects().contains(tempCps) && !doMark) {
				control.clearSelection();
				if (tempCps != null) {
					control.addSelectedObject(tempCps);
				}
			}
			if (dragging) {
				try {
					// tempCps in the upperNode? else its a connected Object from
					// outside
					if (groupNode.getObjectsInThisLayer().anyMatch(obj -> obj.equals(tempCps))) {
						dragged = true;
						float xDist, yDist; // Distance

						x = e.getX();
						y = e.getY();

						// Make sure its in bounds
						if (e.getX() < GuiSettings.getPictureScaleDiv2() + +5)
							x = GuiSettings.getPictureScaleDiv2() + +5;
						else if (e.getX() > this.getWidth() - GuiSettings.getPictureScaleDiv2())
							x = this.getWidth() - GuiSettings.getPictureScaleDiv2();
						if (e.getY() < GuiSettings.getPictureScaleDiv2())
							y = GuiSettings.getPictureScaleDiv2();
						else if (e.getY() > this.getHeight() - GuiSettings.getPictureScaleDiv2())
							y = this.getHeight() - GuiSettings.getPictureScaleDiv2();

						// Distance
						xDist = x - tempCps.getPosition().getX();
						yDist = y - tempCps.getPosition().getY();

						tempCps.setPosition(x, y); // Drag Position

						// TipText Position and name
						toolTip = true;
						toolTipText = tempCps.getName() + ", " + tempCps.getId();
						toolTipPos.setX(tempCps.getPosition().getX() - GuiSettings.getPictureScaleDiv2());
						toolTipPos.setY(tempCps.getPosition().getY() - GuiSettings.getPictureScaleDiv2());

						// All Selected Objects
						for (AbstractCanvasObject cps : GuiSettings.getSelectedObjects()) {
							if (cps != tempCps) {
								x = (int) (cps.getPosition().getX() + xDist);
								y = (int) (cps.getPosition().getY() + yDist);

								// Make sure its in bounds
								if (x < +5 + GuiSettings.getPictureScaleDiv2())
									x = GuiSettings.getPictureScaleDiv2() + +5;
								else if (x > this.getWidth() - GuiSettings.getPictureScaleDiv2())
									x = this.getWidth() - GuiSettings.getPictureScaleDiv2();
								if (y <= GuiSettings.getPictureScaleDiv2())
									y = GuiSettings.getPictureScaleDiv2();
								else if (y > this.getHeight() - GuiSettings.getPictureScaleDiv2())
									y = this.getHeight() - GuiSettings.getPictureScaleDiv2();

								cps.setPosition(x, y);
							}
						}
					}
					/**
					 * check if something would be replaced
					 */
					if (GuiSettings.getSelectedObjects().size() == 1)
						checkForReplacement(groupNode.getObjectsInThisLayer().toList(), tempCps, x, y);

					repaint();
				} catch (Exception eex) {

				}

			}

			// Mark Objects
			if (doMark) {
				tempSelected.clear();
				groupNode.getObjectsInThisLayer().forEach(cps -> 
				{
					int x1 = sx, x2 = x, y1 = sy, y2 = y;

					if (sx >= x) {
						x1 = x;
						x2 = sx;
					}
					if (sy >= y) {
						y1 = y;
						y2 = sy;
					}
					if (x1 <= cps.getPosition().getX() + GuiSettings.getPictureScaleDiv2()
							&& y1 <= cps.getPosition().getY() + GuiSettings.getPictureScaleDiv2() && x2 >= cps.getPosition().getX()
							&& y2 >= cps.getPosition().getY()) {
						tempSelected.add(cps);
					}
				});
			}

			repaint();
		}
	}

	@Override
	public void mouseMoved(MouseEvent e) {
		{
			x = e.getX();
			y = e.getY();
			// Everything for the tooltip :)
			boolean on = false;
			Optional<AbstractCanvasObject> objectUnderCursor = groupNode.getObjectsInThisLayer().filter(cps -> {

				cx = cps.getPosition().getX() - GuiSettings.getPictureScaleDiv2();
				cy = cps.getPosition().getY() - GuiSettings.getPictureScaleDiv2();
				boolean mouseOverObject = x - GuiSettings.getPictureScale() <= cx && y - GuiSettings.getPictureScale() <= cy && x >= cx
						&& y >= cy;
				return mouseOverObject;
			}).findAny();
			on |= objectUnderCursor.isPresent();
			objectUnderCursor.ifPresent(cps -> {
				toolTipPos.setX(cps.getPosition().getX() - GuiSettings.getPictureScaleDiv2());
				toolTipPos.setY(cps.getPosition().getY() + GuiSettings.getPictureScaleDiv2());
				toolTipText = cps.getName() + ", " + cps.getId();
			});

			toolTip = on;
			repaint();
		}
	}
}
